@@ -6,6 +6,8 @@ json 工具
 
 本模块提供 json 和 jsonrpc 工具.
 
+TODO:: 支持 JSONRPC Batched Call
+
 索引
 =================================
 
@@ -142,14 +144,14 @@ class JSONRPCBuilder(object):
     """
 
     @staticmethod
-    def request(method, params = None, id = 0, version = "2.0"):
+    def request(method, params = None, id = None, version = "1.0"):
         """
         构建 jsonrpc 请求
 
         :param method: method name
         :param params: params, list or dict
-        :param id: id
-        :param version: jsonrpc protocol version, defualts is 2.0
+        :param id: id, 
+        :param version: jsonrpc protocol version, defualts is 1.0
 
         :type method: string
         :type id: int
@@ -157,59 +159,55 @@ class JSONRPCBuilder(object):
 
         :rtype: string
         """
-
+        
+        version = version or '1.0'
+        json = {}
         fversion = float(version)
         if fversion >= 2.0:
-            version_field_name = 'jsonrpc'
+            json['jsonrpc'] = version
         else:
-            version_field_name = 'version'    
-        id = int(id)
-        if params:
+            json['version'] = version
+        json['method'] = method
+        if params is not None:
             params_type = type(params)
             if params_type is tuple:
                 params = list(params)
             elif params_type not in [dict, list]:
                 params = [params]
-            json_str = "{\"%(version_field_name)s\":\"%(version)s\", \"method\":\"%(method)s\", \"params\":%(params)s, \"id\":%(id)s}" \
-                        % dict(version_field_name = version_field_name, version = version, method = method, params = dumps(params), id = id)
-        else:
-            json_str = "{\"%(version_field_name)s\":\"%(version)s\", \"method\":\"%(method)s\", \"id\":%(id)s}" \
-                        % dict(version_field_name = version_field_name, version = version, method = method, id = id)
-        return json_str
+            json['params'] = params
+        # request is a Notification
+        if id is not None:
+            json['id'] = id
+        return dumps(json)
 
     @staticmethod
-    def response(result = None, error = None, id = 0, version = "2.0"):
+    def response(result = None, error = None, id = None, version = "1.0"):
         """
         构建 jsonrpc 响应
 
         :param result: python object
         :param error: :class:`JSONRPCError`
         :param id: id
-        :param version: jsonrpc protocol version, defualts is 2.0
+        :param version: jsonrpc protocol version, defualts is 1.0
 
         :rtype: string
         """
         
+        version = version or '1.0'
+        json = {}
         fversion = float(version)
         if fversion >= 2.0:
-            version_field_name = 'jsonrpc'
+            json['jsonrpc'] = version
         else:
-            version_field_name = 'version'
-        if error:
-            if hasattr(error, 'data'):
-                json_str = '''
-                {"%(version_field_name)s":"%(version)s", "result":null, "error":{"code":%(code)s,"message":"%(message)s", "data" : "%(data)s"}, "id":null}
-                ''' % dict(version_field_name = version_field_name, version = version, code = error.code, message = error.message or 'null', data = error.data or 'null')
-            else:
-                json_str = '''
-                {"%(version_field_name)s":"%(version)s", "result":null, "error":{"code":%(code)d,"message":"%(message)s"}, "id":null}
-                ''' % dict(version_field_name = version_field_name, version = version, code = int(error.code), message = error.message or 'null')
+            json['version'] = version
+        if error is not None:
+            json['error'] = {'code' : error.code, 'message' : error.message, 'data' : error.data}
         else:
+            assert id is not None, ValueError('This `id` MUST be an integer')
             id = int(id)
-            json_str = '''
-            {"jsonrpc":"%(version)s", "result":%(result)s, "error":null, "id":%(id)s}
-            ''' % dict(version = version, result = dumps(result), id = id)
-        return json_str
+            json['result'] = result
+        json['id'] = id
+        return dumps(json)
 
 class JSONRPCErrorMeta(type):
 
@@ -301,7 +299,6 @@ def jsonrpc_loads(data, version = None):
     """
 
     jsonrpc_data = loads(data)
-    assert "id" in jsonrpc_data, JSONRPCParseError("`id` field invalid")
     if version:
         fversion = float(version)
         if fversion >= 2.0:
@@ -309,24 +306,21 @@ def jsonrpc_loads(data, version = None):
         else:
             version_field_name = 'version'
         assert version_field_name in jsonrpc_data, JSONRPCParseError("`%s` field missing." % version_field_name)
-    else:
-        assert ('version' in jsonrpc_data) or ('jsonrpc' in jsonrpc_data), JSONRPCParseError("`version` or `jsonrpc` field missing.")
+    # is request
     if "method" in jsonrpc_data and isinstance(jsonrpc_data["method"], basestring):
         jsonrpc_data["method"] = jsonrpc_data["method"].strip()
         assert jsonrpc_data["method"], JSONRPCParseError("`method` field invalid.")
         if "params" in jsonrpc_data:
-            assert type(jsonrpc_data["params"]) in [list, dict], JSONRPCParseError("`params` field invalid.")
+            assert type(jsonrpc_data["params"]) in [tuple, set, list, dict], JSONRPCParseError("`params` field invalid.")
     else:
-        assert "result" in jsonrpc_data, JSONRPCParseError("`result` field missing")
-        assert "error" in jsonrpc_data, JSONRPCParseError("`error` field missing")
-        if jsonrpc_data["result"]:
-            assert jsonrpc_data["error"] == None, JSONRPCParseError("`error` field must be `null`")
+        if 'result' in jsonrpc_data:
+            assert 'error' not in jsonrpc_data, JSONRPCParseError("`error` field must be `null`")
+            assert ('id' in jsonrpc_data) and jsonrpc_data["id"] != None, JSONRPCParseError("`id` field missing or invalid.")
+        elif 'error' in jsonrpc_data:
+            assert "code" in jsonrpc_data["error"] and (type(jsonrpc_data["error"]["code"]) in [int]) \
+                and "message" in jsonrpc_data["error"], JSONRPCParseError("`code` or `messgae` fields invalid.")
         else:
-            if jsonrpc_data["error"]:
-                assert jsonrpc_data["id"] == None, JSONRPCParseError("`id` field must be null.")
-                assert "code" in jsonrpc_data["error"] and (type(jsonrpc_data["error"]["code"]) is int) \
-                    and "message" in jsonrpc_data["error"] \
-                    and isinstance(jsonrpc_data["error"]["message"], basestring), JSONRPCParseError("`code` or `messgae` fields invalid.")
+            raise JSONRPCParseError("`result` or `error` field missing")
     return jsonrpc_data
 
 class JSONRPCService(DictMixin, object):
@@ -395,14 +389,23 @@ class JSONRPCService(DictMixin, object):
         elif 'jsonrpc' in jsonrpc_data:
             version = jsonrpc_data['jsonrpc']
         else:
-            self._logger.debug("the 'version' or 'jsonrpc' field missing." % post_data, exc_info = True)
-            resp = Response(JSONRPCBuilder.response(error = JSONRPCParseError()),
-                            content_type = JSON_MIME_TYPE, charset = 'utf-8')
-            return resp(environ, start_response)
+            # 如果请求里没有指定 jsonrpc 协议版本, 则默认为 1.0
+            version = '1.0'
+        if 'id' in jsonrpc_data:
+            id = jsonrpc_data['id']
+            is_notification = False
+        else:
+            id = None
+            is_notification = True
+            self._logger.debug("jsonrpc request is a notification.")
         if "method" not in jsonrpc_data:
             self._logger.debug("`method` field not found.")
-            resp = Response(JSONRPCBuilder.response(error = JSONRPCParseError(), version = version),
-                            content_type = JSON_MIME_TYPE, charset = 'utf-8')
+            # is a notification
+            if is_notification:
+                resp = Response()
+            else:
+                resp = Response(JSONRPCBuilder.response(error = JSONRPCParseError(), version = version, id = id),
+                                content_type = JSON_MIME_TYPE, charset = 'utf-8')
             return resp(environ, start_response)
         args = []
         kargs = {}
@@ -422,8 +425,11 @@ class JSONRPCService(DictMixin, object):
         provider = self.method_resolve(jsonrpc_data["method"])
         if provider is None:
             self._logger.debug("method '%s' not found." % jsonrpc_data["method"])
-            resp = Response(JSONRPCBuilder.response(error = JSONRPCMethodNotFoundError(), version = version),
-                            content_type = JSON_MIME_TYPE, charset = 'utf-8')
+            if is_notification:
+                resp = Response()
+            else:
+                resp = Response(JSONRPCBuilder.response(error = JSONRPCMethodNotFoundError(), version = version, id = id),
+                                content_type = JSON_MIME_TYPE, charset = 'utf-8')
             return resp(environ, start_response)
         
         environ["khan.web.service"] = self
@@ -432,8 +438,11 @@ class JSONRPCService(DictMixin, object):
                            % (self.get_provider_name(provider), args, kargs))
         try:
             result = provider(*args, **kargs)
-            resp = Response(JSONRPCBuilder.response(result = result, id = jsonrpc_data["id"], version = version),
-                            content_type = JSON_MIME_TYPE, charset = 'utf-8')
+            if is_notification:
+                resp = Response()
+            else:
+                resp = Response(JSONRPCBuilder.response(result = result, id = id, version = version),
+                                content_type = JSON_MIME_TYPE, charset = 'utf-8')
             return resp(environ, start_response)
         except HTTPException, httpe:
             self._logger.debug("`%r` raised HTTPException: %s (code: %s)",
@@ -445,20 +454,29 @@ class JSONRPCService(DictMixin, object):
             return httpe(environ, start_response)
         except TypeError, e:
             self._logger.error("calling `%r` with invalid arguments." % self.get_provider_name(provider), exc_info = True)
-            resp = Response(JSONRPCBuilder.response(error = JSONRPCInvalidParamsError(), 
-                                                    id = jsonrpc_data["id"], version = version),
-                            content_type = JSON_MIME_TYPE, charset = 'utf-8')
+            if is_notification:
+                resp = Response()
+            else:
+                resp = Response(JSONRPCBuilder.response(error = JSONRPCInvalidParamsError(), 
+                                                        id = id, version = version),
+                                content_type = JSON_MIME_TYPE, charset = 'utf-8')
             return resp(environ, start_response)
         except JSONRPCError, e:
             self._logger.error("jsonrpc error.", exc_info = True)
-            resp = Response(JSONRPCBuilder.response(error = e, id = jsonrpc_data["id"], version = version),
-                            content_type = JSON_MIME_TYPE, charset = 'utf-8')
+            if is_notification:
+                resp = Response()
+            else:
+                resp = Response(JSONRPCBuilder.response(error = e, id = id, version = version),
+                                content_type = JSON_MIME_TYPE, charset = 'utf-8')
             return resp(environ, start_response)
         except:
             self._logger.error("`%r` raised unknown exception." % self.get_provider_name(provider), exc_info = True)
-            resp = Response(JSONRPCBuilder.response(error = JSONRPCInternalError(), 
-                                                    id = jsonrpc_data["id"], version = version),
-                            content_type = JSON_MIME_TYPE, charset = 'utf-8')
+            if is_notification:
+                resp = Response()
+            else:
+                resp = Response(JSONRPCBuilder.response(error = JSONRPCInternalError(), 
+                                                        id = id, version = version),
+                                content_type = JSON_MIME_TYPE, charset = 'utf-8')
             return resp(environ, start_response)
 
     def __call__(self, environ, start_response):
@@ -500,7 +518,8 @@ class JSONRPCClient(object):
             "Accept": "text/plain"
     }
 
-    def __init__(self, url, timeout = None, service_name = None, http_headers = None, version = "2.0", http_client = None, id = 0):
+    def __init__(self, url, timeout = None, service_name = None, 
+                 http_headers = None, version = "2.0", http_client = None, id = 0):
         """
         :param url: jsonrpc 服务 url, 支持 https.
         :param timeout: socket 级的 timeout(秒) 设置.
@@ -550,7 +569,7 @@ class JSONRPCClient(object):
             raise JSONRPCParseError("%s" % e)
         if jsonrpc_data['id'] != self._id:
             raise JSONRPCServerError('JSONRPC request id is `%s`, but response id is `%s`' % (self._id, jsonrpc_data['id']))
-        if (jsonrpc_data["error"] != None):
+        if 'error' in jsonrpc_data and jsonrpc_data["error"] != None:
             if jsonrpc_data["error"]["code"] in jsonrpc_error_map:
                 err_cls = jsonrpc_error_map[jsonrpc_data["error"]["code"]](jsonrpc_data["error"]["message"],
                                                                                data =  jsonrpc_data["error"].get("data", None))
@@ -558,8 +577,10 @@ class JSONRPCClient(object):
                 err_cls = JSONRPCError(jsonrpc_data["error"]["message"], jsonrpc_data["error"]["code"],
                                        data = jsonrpc_data["error"].get("data", None))
             raise err_cls
-        else:
+        elif 'result' in jsonrpc_data:
             return jsonrpc_data["result"]
+        else:
+            raise JSONRPCServerError('response invalid')
 
 class JSONPProxy(object):
     """
